@@ -20,9 +20,10 @@ import unittest
 import mock
 
 from opencensus.ext.azure import trace_exporter
+from opencensus.ext.azure.common.transport import TransportStatusCode
 from opencensus.trace.link import Link
 
-TEST_FOLDER = os.path.abspath('.test.exporter')
+TEST_FOLDER = os.path.abspath('.test.trace.exporter')
 
 
 def setUpModule():
@@ -100,14 +101,14 @@ class TestAzureExporter(unittest.TestCase):
         exporter._stop()
 
     @mock.patch('opencensus.ext.azure.trace_exporter.AzureExporter.span_data_to_envelope')  # noqa: E501
-    def test_emit_failure(self, span_data_to_envelope_mock):
+    def test_emit_retry(self, span_data_to_envelope_mock):
         span_data_to_envelope_mock.return_value = ['bar']
         exporter = trace_exporter.AzureExporter(
             instrumentation_key='12345678-1234-5678-abcd-12345678abcd',
             storage_path=os.path.join(TEST_FOLDER, self.id()),
         )
         with mock.patch('opencensus.ext.azure.trace_exporter.AzureExporter._transmit') as transmit:  # noqa: E501
-            transmit.return_value = 10
+            transmit.return_value = TransportStatusCode.RETRY
             exporter.emit(['foo'])
         self.assertEqual(len(os.listdir(exporter.storage.path)), 1)
         self.assertIsNone(exporter.storage.get())
@@ -122,7 +123,7 @@ class TestAzureExporter(unittest.TestCase):
             storage_path=os.path.join(TEST_FOLDER, self.id()),
         )
         with mock.patch('opencensus.ext.azure.trace_exporter.AzureExporter._transmit') as transmit:  # noqa: E501
-            transmit.return_value = 0
+            transmit.return_value = TransportStatusCode.SUCCESS
             exporter.emit([])
             exporter.emit(['foo'])
             self.assertEqual(len(os.listdir(exporter.storage.path)), 0)
@@ -497,7 +498,7 @@ class TestAzureExporter(unittest.TestCase):
             'RequestData')
 
         # SpanKind.SERVER HTTP - with exceptions
-        envelopes = exporter.span_data_to_envelope(SpanData(
+        envelopes = list(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -528,9 +529,10 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
-        ))
+        )))
+        self.assertEqual(len(envelopes), 2)
 
-        envelope = next(envelopes)
+        envelope = envelopes[0]
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -550,7 +552,7 @@ class TestAzureExporter(unittest.TestCase):
             envelope.data.baseType,
             'ExceptionData')
 
-        envelope = next(envelopes)
+        envelope = envelopes[1]
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
