@@ -393,6 +393,15 @@ class TestDerivedGauge(unittest.TestCase):
         unused_mock_fn.assert_not_called()
         self.assertEqual(len(derived_gauge.points.keys()), 1)
 
+        # with kwargs
+        def fn_with_args(value=None):
+            if value:
+                return value
+            return 0
+        label_values2 = [1, 2]
+        point3 = derived_gauge.create_time_series(label_values2, fn_with_args, value=5)  # noqa: E501
+        self.assertEqual(point3.get_value(), 5)
+
     def test_create_default_time_series(self):
         derived_gauge = gauge.DerivedLongGauge(
             Mock(), Mock(), Mock(), [Mock(), Mock])
